@@ -1,39 +1,39 @@
 /****************************************************************************
 **
-** Copyright (C) 2012 Nokia Corporation and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the QtGui module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
+** Software or, alternatively, in accordance with the terms contained in
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see http://www.qt.io/terms-conditions. For further
+** information use the contact form at http://www.qt.io/contact-us.
+**
 ** GNU Lesser General Public License Usage
-** This file may be used under the terms of the GNU Lesser General Public
-** License version 2.1 as published by the Free Software Foundation and
-** appearing in the file LICENSE.LGPL included in the packaging of this
-** file. Please review the following information to ensure the GNU Lesser
-** General Public License version 2.1 requirements will be met:
+** Alternatively, this file may be used under the terms of the GNU Lesser
+** General Public License version 2.1 or version 3 as published by the Free
+** Software Foundation and appearing in the file LICENSE.LGPLv21 and
+** LICENSE.LGPLv3 included in the packaging of this file. Please review the
+** following information to ensure the GNU Lesser General Public License
+** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
 ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Nokia gives you certain additional
-** rights. These rights are described in the Nokia Qt LGPL Exception
+** As a special exception, The Qt Company gives you certain additional
+** rights. These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU General
-** Public License version 3.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of this
-** file. Please review the following information to ensure the GNU General
-** Public License version 3.0 requirements will be met:
-** http://www.gnu.org/copyleft/gpl.html.
-**
-** Other Usage
-** Alternatively, this file may be used in accordance with the terms and
-** conditions contained in a signed written agreement between you and Nokia.
-**
-**
-**
-**
-**
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3.0 as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU General Public License version 3.0 requirements will be
+** met: http://www.gnu.org/copyleft/gpl.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -179,7 +179,7 @@ struct Blend_ARGB32_on_RGB16_SourceAndConstAlpha {
 };
 
 void qt_scale_image_rgb16_on_rgb16(uchar *destPixels, int dbpl,
-                                   const uchar *srcPixels, int sbpl,
+                                   const uchar *srcPixels, int sbpl, int sh,
                                    const QRectF &targetRect,
                                    const QRectF &sourceRect,
                                    const QRect &clip,
@@ -194,17 +194,17 @@ void qt_scale_image_rgb16_on_rgb16(uchar *destPixels, int dbpl,
 #endif
     if (const_alpha == 256) {
         Blend_RGB16_on_RGB16_NoAlpha noAlpha;
-        qt_scale_image_16bit<quint16>(destPixels, dbpl, srcPixels, sbpl,
+        qt_scale_image_16bit<quint16>(destPixels, dbpl, srcPixels, sbpl, sh,
                                       targetRect, sourceRect, clip, noAlpha);
     } else {
         Blend_RGB16_on_RGB16_ConstAlpha constAlpha(const_alpha);
-        qt_scale_image_16bit<quint16>(destPixels, dbpl, srcPixels, sbpl,
+        qt_scale_image_16bit<quint16>(destPixels, dbpl, srcPixels, sbpl, sh,
                                      targetRect, sourceRect, clip, constAlpha);
     }
 }
 
 void qt_scale_image_argb24_on_rgb16(uchar *destPixels, int dbpl,
-                                   const uchar *srcPixels, int sbpl,
+                                   const uchar *srcPixels, int sbpl, int sh,
                                    const QRectF &targetRect,
                                    const QRectF &sourceRect,
                                    const QRect &clip,
@@ -219,18 +219,18 @@ void qt_scale_image_argb24_on_rgb16(uchar *destPixels, int dbpl,
 #endif
     if (const_alpha == 256) {
         Blend_ARGB24_on_RGB16_SourceAlpha noAlpha;
-        qt_scale_image_16bit<qargb8565>(destPixels, dbpl, srcPixels, sbpl,
+        qt_scale_image_16bit<qargb8565>(destPixels, dbpl, srcPixels, sbpl, sh,
                                         targetRect, sourceRect, clip, noAlpha);
     } else {
         Blend_ARGB24_on_RGB16_SourceAndConstAlpha constAlpha(const_alpha);
-        qt_scale_image_16bit<qargb8565>(destPixels, dbpl, srcPixels, sbpl,
+        qt_scale_image_16bit<qargb8565>(destPixels, dbpl, srcPixels, sbpl, sh,
                                         targetRect, sourceRect, clip, constAlpha);
     }
 }
 
 
 void qt_scale_image_argb32_on_rgb16(uchar *destPixels, int dbpl,
-                                    const uchar *srcPixels, int sbpl,
+                                    const uchar *srcPixels, int sbpl, int sh,
                                     const QRectF &targetRect,
                                     const QRectF &sourceRect,
                                     const QRect &clip,
@@ -245,11 +245,11 @@ void qt_scale_image_argb32_on_rgb16(uchar *destPixels, int dbpl,
 #endif
     if (const_alpha == 256) {
         Blend_ARGB32_on_RGB16_SourceAlpha noAlpha;
-        qt_scale_image_16bit<quint32>(destPixels, dbpl, srcPixels, sbpl,
+        qt_scale_image_16bit<quint32>(destPixels, dbpl, srcPixels, sbpl, sh,
                                       targetRect, sourceRect, clip, noAlpha);
     } else {
         Blend_ARGB32_on_RGB16_SourceAndConstAlpha constAlpha(const_alpha);
-        qt_scale_image_16bit<quint32>(destPixels, dbpl, srcPixels, sbpl,
+        qt_scale_image_16bit<quint32>(destPixels, dbpl, srcPixels, sbpl, sh,
                                      targetRect, sourceRect, clip, constAlpha);
     }
 }
@@ -611,7 +611,7 @@ struct Blend_ARGB32_on_ARGB32_SourceAndConstAlpha {
 };
 
 void qt_scale_image_rgb32_on_rgb32(uchar *destPixels, int dbpl,
-                                   const uchar *srcPixels, int sbpl,
+                                   const uchar *srcPixels, int sbpl, int sh,
                                    const QRectF &targetRect,
                                    const QRectF &sourceRect,
                                    const QRect &clip,
@@ -626,17 +626,17 @@ void qt_scale_image_rgb32_on_rgb32(uchar *destPixels, int dbpl,
 #endif
     if (const_alpha == 256) {
         Blend_RGB32_on_RGB32_NoAlpha noAlpha;
-        qt_scale_image_32bit(destPixels, dbpl, srcPixels, sbpl,
+        qt_scale_image_32bit(destPixels, dbpl, srcPixels, sbpl, sh,
                              targetRect, sourceRect, clip, noAlpha);
     } else {
         Blend_RGB32_on_RGB32_ConstAlpha constAlpha(const_alpha);
-        qt_scale_image_32bit(destPixels, dbpl, srcPixels, sbpl,
+        qt_scale_image_32bit(destPixels, dbpl, srcPixels, sbpl, sh,
                              targetRect, sourceRect, clip, constAlpha);
     }
 }
 
 void qt_scale_image_argb32_on_argb32(uchar *destPixels, int dbpl,
-                                     const uchar *srcPixels, int sbpl,
+                                     const uchar *srcPixels, int sbpl, int sh,
                                      const QRectF &targetRect,
                                      const QRectF &sourceRect,
                                      const QRect &clip,
@@ -651,11 +651,11 @@ void qt_scale_image_argb32_on_argb32(uchar *destPixels, int dbpl,
 #endif
     if (const_alpha == 256) {
         Blend_ARGB32_on_ARGB32_SourceAlpha sourceAlpha;
-        qt_scale_image_32bit(destPixels, dbpl, srcPixels, sbpl,
+        qt_scale_image_32bit(destPixels, dbpl, srcPixels, sbpl, sh,
                              targetRect, sourceRect, clip, sourceAlpha);
     } else {
         Blend_ARGB32_on_ARGB32_SourceAndConstAlpha constAlpha(const_alpha);
-        qt_scale_image_32bit(destPixels, dbpl, srcPixels, sbpl,
+        qt_scale_image_32bit(destPixels, dbpl, srcPixels, sbpl, sh,
                              targetRect, sourceRect, clip, constAlpha);
     }
 }
