@@ -1,39 +1,39 @@
 /****************************************************************************
 **
-** Copyright (C) 2012 Nokia Corporation and/or its subsidiary(-ies).
-** Contact: http://www.qt-project.org/
+** Copyright (C) 2015 The Qt Company Ltd.
+** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the QtNetwork module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
+** Software or, alternatively, in accordance with the terms contained in
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see http://www.qt.io/terms-conditions. For further
+** information use the contact form at http://www.qt.io/contact-us.
+**
 ** GNU Lesser General Public License Usage
-** This file may be used under the terms of the GNU Lesser General Public
-** License version 2.1 as published by the Free Software Foundation and
-** appearing in the file LICENSE.LGPL included in the packaging of this
-** file. Please review the following information to ensure the GNU Lesser
-** General Public License version 2.1 requirements will be met:
+** Alternatively, this file may be used under the terms of the GNU Lesser
+** General Public License version 2.1 or version 3 as published by the Free
+** Software Foundation and appearing in the file LICENSE.LGPLv21 and
+** LICENSE.LGPLv3 included in the packaging of this file. Please review the
+** following information to ensure the GNU Lesser General Public License
+** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
 ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Nokia gives you certain additional
-** rights. These rights are described in the Nokia Qt LGPL Exception
+** As a special exception, The Qt Company gives you certain additional
+** rights. These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU General
-** Public License version 3.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of this
-** file. Please review the following information to ensure the GNU General
-** Public License version 3.0 requirements will be met:
-** http://www.gnu.org/copyleft/gpl.html.
-**
-** Other Usage
-** Alternatively, this file may be used in accordance with the terms and
-** conditions contained in a signed written agreement between you and Nokia.
-**
-**
-**
-**
-**
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3.0 as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU General Public License version 3.0 requirements will be
+** met: http://www.gnu.org/copyleft/gpl.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -141,7 +141,7 @@ QAuthenticator::QAuthenticator()
 */
 QAuthenticator::~QAuthenticator()
 {
-    if (d && !d->ref.deref())
+    if (d)
         delete d;
 }
 
@@ -149,10 +149,10 @@ QAuthenticator::~QAuthenticator()
     Constructs a copy of \a other.
 */
 QAuthenticator::QAuthenticator(const QAuthenticator &other)
-    : d(other.d)
+    : d(0)
 {
-    if (d)
-        d->ref.ref();
+    if (other.d)
+        *this = other;
 }
 
 /*!
@@ -163,12 +163,23 @@ QAuthenticator &QAuthenticator::operator=(const QAuthenticator &other)
     if (d == other.d)
         return *this;
 
-    if (d && !d->ref.deref())
+    // Do not share the d since challenge reponse/based changes
+    // could corrupt the internal store and different network requests
+    // can utilize different types of proxies.
+    detach();
+    if (other.d) {
+        d->user = other.d->user;
+        d->userDomain = other.d->userDomain;
+        d->workstation = other.d->workstation;
+        d->extractedUser = other.d->extractedUser;
+        d->password = other.d->password;
+        d->realm = other.d->realm;
+        d->method = other.d->method;
+        d->options = other.d->options;
+    } else if (d->phase == QAuthenticatorPrivate::Start) {
         delete d;
-
-    d = other.d;
-    if (d)
-        d->ref.ref();
+        d = 0;
+    }
     return *this;
 }
 
@@ -208,28 +219,8 @@ QString QAuthenticator::user() const
 void QAuthenticator::setUser(const QString &user)
 {
     detach();
-    int separatorPosn = 0;
-
-    switch(d->method) {
-    case QAuthenticatorPrivate::Ntlm:
-        if((separatorPosn = user.indexOf(QLatin1String("\\"))) != -1) {
-            //domain name is present
-            d->realm.clear();
-            d->userDomain = user.left(separatorPosn);
-            d->extractedUser = user.mid(separatorPosn + 1);
-            d->user = user;
-        } else {
-            d->extractedUser = user;
-            d->user = user;
-	    d->realm.clear();
-            d->userDomain.clear();
-        }
-        break;
-    default:
-        d->user = user;
-        d->userDomain.clear();
-        break;
-    }
+    d->user = user;
+    d->updateCredentials();
 }
 
 /*!
@@ -256,12 +247,11 @@ void QAuthenticator::detach()
 {
     if (!d) {
         d = new QAuthenticatorPrivate;
-        d->ref = 1;
         return;
     }
 
-    qAtomicDetach(d);
-    d->phase = QAuthenticatorPrivate::Start;
+    if (d->phase == QAuthenticatorPrivate::Done)
+        d->phase = QAuthenticatorPrivate::Start;
 }
 
 /*!
@@ -322,8 +312,7 @@ bool QAuthenticator::isNull() const
 }
 
 QAuthenticatorPrivate::QAuthenticatorPrivate()
-    : ref(0)
-    , method(None)
+    : method(None)
     , hasFailed(false)
     , phase(Start)
     , nonceCount(0)
@@ -348,6 +337,33 @@ void QAuthenticatorPrivate::parseHttpResponse(const QHttpResponseHeader &header,
 }
 #endif
 
+QAuthenticatorPrivate::~QAuthenticatorPrivate()
+{
+}
+
+void QAuthenticatorPrivate::updateCredentials()
+{
+    int separatorPosn = 0;
+
+    switch (method) {
+    case QAuthenticatorPrivate::Ntlm:
+        if ((separatorPosn = user.indexOf(QLatin1String("\\"))) != -1) {
+            //domain name is present
+            realm.clear();
+            userDomain = user.left(separatorPosn);
+            extractedUser = user.mid(separatorPosn + 1);
+        } else {
+            extractedUser = user;
+            realm.clear();
+            userDomain.clear();
+        }
+        break;
+    default:
+        userDomain.clear();
+        break;
+    }
+}
+
 void QAuthenticatorPrivate::parseHttpResponse(const QList<QPair<QByteArray, QByteArray> > &values, bool isProxy)
 {
     const char *search = isProxy ? "proxy-authenticate" : "www-authenticate";
@@ -381,6 +397,8 @@ void QAuthenticatorPrivate::parseHttpResponse(const QList<QPair<QByteArray, QByt
         }
     }
 
+    // Reparse credentials since we know the method now
+    updateCredentials();
     challenge = headerVal.trimmed();
     QHash<QByteArray, QByteArray> options = parseDigestAuthenticationChallenge(challenge);
 
