@@ -1,7 +1,7 @@
 /****************************************************************************
 **
 ** Copyright (C) 2012 Research In Motion
-** Contact: http://www.qt-project.org/legal
+** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the plugins of the Qt Toolkit.
 **
@@ -10,20 +10,21 @@
 ** Licensees holding valid commercial Qt licenses may use this file in
 ** accordance with the commercial license agreement provided with the
 ** Software or, alternatively, in accordance with the terms contained in
-** a written agreement between you and Digia.  For licensing terms and
-** conditions see http://qt.digia.com/licensing.  For further information
-** use the contact form at http://qt.digia.com/contact-us.
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see http://www.qt.io/terms-conditions. For further
+** information use the contact form at http://www.qt.io/contact-us.
 **
 ** GNU Lesser General Public License Usage
 ** Alternatively, this file may be used under the terms of the GNU Lesser
-** General Public License version 2.1 as published by the Free Software
-** Foundation and appearing in the file LICENSE.LGPL included in the
-** packaging of this file.  Please review the following information to
-** ensure the GNU Lesser General Public License version 2.1 requirements
-** will be met: http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
+** General Public License version 2.1 or version 3 as published by the Free
+** Software Foundation and appearing in the file LICENSE.LGPLv21 and
+** LICENSE.LGPLv3 included in the packaging of this file. Please review the
+** following information to ensure the GNU Lesser General Public License
+** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
+** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Digia gives you certain additional
-** rights.  These rights are described in the Digia Qt LGPL Exception
+** As a special exception, The Qt Company gives you certain additional
+** rights. These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
@@ -34,7 +35,6 @@
 ** ensure the GNU General Public License version 3.0 requirements will be
 ** met: http://www.gnu.org/copyleft/gpl.html.
 **
-**
 ** $QT_END_LICENSE$
 **
 ****************************************************************************/
@@ -45,6 +45,7 @@
 #include <QDebug>
 #include <QThreadStorage>
 #include <QStringList>
+#include <QTimer>
 
 #include <bps/netstatus.h>
 
@@ -369,6 +370,9 @@ void QBBEngine::updateConfiguration(const char *interface)
             changed = true;
         }
 
+        const netstatus_ip_status_t oldIpStatus = ptr->oldIpStatus;
+        ptr->oldIpStatus = ipStatus;
+
         ptrLocker.unlock();
 
         locker.unlock();
@@ -378,7 +382,17 @@ void QBBEngine::updateConfiguration(const char *interface)
 
             Q_EMIT configurationChanged(ptr);
         } else {
+            // maybe Wifi has changed but gateway not yet ready etc.
             qBearerDebug() << Q_FUNC_INFO << "configuration has not changed.";
+            if (oldIpStatus != ipStatus) { // if IP status changed
+                if (ipStatus != NETSTATUS_IP_STATUS_OK
+                        && ipStatus != NETSTATUS_IP_STATUS_ERROR_NOT_UP
+                        && ipStatus != NETSTATUS_IP_STATUS_ERROR_NOT_CONFIGURED) {
+                    // work around race condition in netstatus API by just checking
+                    // again in 300 ms
+                    QTimer::singleShot(300, this, SLOT(doRequestUpdate()));
+                }
+            }
         }
 
         return;
