@@ -1,35 +1,39 @@
 /****************************************************************************
 **
 ** Copyright (C) 2011 - 2012 Research In Motion <blackberry-qt@qnx.com>
-** Contact: http://www.qt-project.org/
+** Contact: http://www.qt.io/licensing/
 **
 ** This file is part of the QtCore module of the Qt Toolkit.
 **
 ** $QT_BEGIN_LICENSE:LGPL$
+** Commercial License Usage
+** Licensees holding valid commercial Qt licenses may use this file in
+** accordance with the commercial license agreement provided with the
+** Software or, alternatively, in accordance with the terms contained in
+** a written agreement between you and The Qt Company. For licensing terms
+** and conditions see http://www.qt.io/terms-conditions. For further
+** information use the contact form at http://www.qt.io/contact-us.
+**
 ** GNU Lesser General Public License Usage
-** This file may be used under the terms of the GNU Lesser General Public
-** License version 2.1 as published by the Free Software Foundation and
-** appearing in the file LICENSE.LGPL included in the packaging of this
-** file. Please review the following information to ensure the GNU Lesser
-** General Public License version 2.1 requirements will be met:
+** Alternatively, this file may be used under the terms of the GNU Lesser
+** General Public License version 2.1 or version 3 as published by the Free
+** Software Foundation and appearing in the file LICENSE.LGPLv21 and
+** LICENSE.LGPLv3 included in the packaging of this file. Please review the
+** following information to ensure the GNU Lesser General Public License
+** requirements will be met: https://www.gnu.org/licenses/lgpl.html and
 ** http://www.gnu.org/licenses/old-licenses/lgpl-2.1.html.
 **
-** In addition, as a special exception, Nokia gives you certain additional
-** rights. These rights are described in the Nokia Qt LGPL Exception
+** As a special exception, The Qt Company gives you certain additional
+** rights. These rights are described in The Qt Company LGPL Exception
 ** version 1.1, included in the file LGPL_EXCEPTION.txt in this package.
 **
 ** GNU General Public License Usage
-** Alternatively, this file may be used under the terms of the GNU General
-** Public License version 3.0 as published by the Free Software Foundation
-** and appearing in the file LICENSE.GPL included in the packaging of this
-** file. Please review the following information to ensure the GNU General
-** Public License version 3.0 requirements will be met:
-** http://www.gnu.org/copyleft/gpl.html.
-**
-**
-**
-**
-**
+** Alternatively, this file may be used under the terms of the GNU
+** General Public License version 3.0 as published by the Free Software
+** Foundation and appearing in the file LICENSE.GPL included in the
+** packaging of this file.  Please review the following information to
+** ensure the GNU General Public License version 3.0 requirements will be
+** met: http://www.gnu.org/copyleft/gpl.html.
 **
 ** $QT_END_LICENSE$
 **
@@ -39,6 +43,8 @@
 
 #include "qbbscreeneventhandler.h"
 
+#include "qbbscreen.h"
+#include "qbbintegration.h"
 #include "qbbinputcontext.h"
 #include "qbbkeytranslator.h"
 
@@ -50,8 +56,9 @@
 
 QT_BEGIN_NAMESPACE
 
-QBBScreenEventHandler::QBBScreenEventHandler()
-    : mLastButtonState(Qt::NoButton)
+QBBScreenEventHandler::QBBScreenEventHandler(QBBIntegration *integration)
+    : mBBIntegration(integration)
+    , mLastButtonState(Qt::NoButton)
     , mLastMouseWindow(0)
 {
     // initialize array of touch points
@@ -109,6 +116,10 @@ bool QBBScreenEventHandler::handleEvent(screen_event_t event, int qnxType)
         handleCreateEvent(event);
         break;
 
+    case SCREEN_EVENT_DISPLAY:
+        handleDisplayEvent(event);
+        break;
+
     default:
         // event ignored
 #if defined(QBBSCREENEVENTHANDLER_DEBUG)
@@ -398,6 +409,8 @@ void QBBScreenEventHandler::handleTouchEvent(screen_event_t event, int qnxType)
         qFatal("QBB: failed to query event position, errno=%d", errno);
     }
 
+    QCursor::setPos(pos[0], pos[1]);
+
     // get window coordinates of touch
     errno = 0;
     int windowPos[2];
@@ -536,6 +549,33 @@ void QBBScreenEventHandler::handleCreateEvent(screen_event_t event)
     emit newWindowCreated(window);
 }
 
+void QBBScreenEventHandler::handleDisplayEvent(screen_event_t event)
+{
+    screen_display_t nativeDisplay = 0;
+    if (screen_get_event_property_pv(event, SCREEN_PROPERTY_DISPLAY, (void **)&nativeDisplay) != 0) {
+        qWarning("QBB: failed to query display property, errno=%d", errno);
+        return;
+    }
+
+    int isAttached = 0;
+    if (screen_get_event_property_iv(event, SCREEN_PROPERTY_ATTACHED, &isAttached) != 0) {
+        qWarning("QBB: failed to query display attached property, errno=%d", errno);
+        return;
+    }
+
+#if defined(QBBSCREENEVENTHANDLER_DEBUG)
+    qDebug() << Q_FUNC_INFO << "display attachment is now:" << isAttached;
+#endif
+    QBBScreen *screen = mBBIntegration->screenForNative(nativeDisplay);
+    if (!screen) {
+        if (isAttached)
+            mBBIntegration->createDisplay(nativeDisplay, false /* not primary, we assume */);
+    } else if (!isAttached) {
+        // libscreen display is deactivated, let's remove the QBBScreen / QScreen
+        mBBIntegration->removeDisplay(screen);
+    }
+}
+
 #include "moc_qbbscreeneventhandler.cpp"
 
 QT_END_NAMESPACE
